@@ -312,15 +312,3 @@ col_2 <- function(...){
 col_1 <- function(...){
   column(1, ...)
 }
-
-
-# Cierra un menú según su tabName
-close.menu <- function(tabname = NA, valor = T) {
-  select <- paste0("a[href^='#shiny-tab-", tabname, "']")
-  if(valor){
-    shinyjs::hide(selector = "ul.menu-open")
-    shinyjs::disable(selector = select)
-  } else {
-    shinyjs::enable(selector = select)
-  }
-}
\ No newline at end of file
