@@ -109,7 +109,8 @@ mod_ind_nuevos_server <- function(input, output, session, newCases, updateData2,
   
   observeEvent(codedioma$idioma, {
     
-    nombres <- list( "knn", "svm", "dt", "rf", "boost","nn", "rl", "rlr", "rd")
+    # "knn"= kvecinos , "svm"= maquinas de soporte vectorial, "dt"= árboles de decisión, "rf"= Bosques aleatorios, "boost"= potenciacion,"nn"= redes neuronales, "rl"= regresion lineal, "rlr"= regresion lineal penalizada, "rd" = reducción de la dimension
+    nombres <- list( "knn", "svm", "dt", "rf", "boost","nn", "rl", "rlr", "rd") 
     names(nombres) <- tr(c("knn", "svm", "dt", "rf", "boost",  "nn", "rl", "rlr", "rd"),codedioma$idioma)
     
     updateSelectInput(session, "selectModelsPred", choices = nombres, selected = input$selectModelsPred)
@@ -207,10 +208,10 @@ mod_ind_nuevos_server <- function(input, output, session, newCases, updateData2,
                           shrinkage    <- isolate(input$shrinkage_boosting)
                           gen.code     <- codeBoost(variable, n.trees, distribution, shrinkage)
                           isolate(codedioma$code <- append(codedioma$code,  paste0(sub, gen.code)))
-                          modelo <- traineR::train.gbm(as.formula(var), data = datos, distribution = k, n.trees = n.trees, shrinkage = shrinkage)
+                          isolate(modelo <- traineR::train.gbm(as.formula(var), data = train, distribution = distribution, n.trees = n.trees, shrinkage = shrinkage))
                           isolate(modelo)
                         },
-                        nn    = {
+                        nn = {
                           threshold  <- isolate(input$threshold.nn.pred)
                           stepmax    <- isolate(input$stepmax.nn.pred)
                           capas.np   <- c(isolate(input$nn_cap_1),isolate(input$nn_cap_2),
@@ -220,18 +221,25 @@ mod_ind_nuevos_server <- function(input, output, session, newCases, updateData2,
                                           isolate(input$nn_cap_9),isolate(input$nn_cap_10))
                           cant.capas <- isolate(input$cant.capas.nn.pred)
                           capas.np   <<- as.vector(as.numeric(capas.np[1:cant.capas]))
+                          gen.code <- codeNn(variable, capas.np, threshold, stepmax)
+                          isolate(codedioma$code <- append(codedioma$code,  paste0(sub, gen.code)))
                           
-                          isolate(modelo     <- traineR::train.neuralnet(
+                          # --- NUEVO BLOQUE: guardar estructura del model.matrix ---
+                          # Expandir los factores del entrenamiento
+                          mm_train <- model.matrix(as.formula(var), data = train)
+                          mm_train <- as.data.frame(mm_train)
+                          newCases$model.matrix.template <- colnames(mm_train)  # guardar nombres de columnas
+                          
+                          isolate(modelo <- traineR::train.neuralnet(
                             formula   = as.formula(var),
                             data      = train,
                             threshold = threshold,
                             stepmax   = stepmax,
-                            hidden    = capas.np))
-                          gen.code <- codeNn(variable, capas.np, threshold, stepmax)
-                          
-                          isolate(codedioma$code <- append(codedioma$code,  paste0(sub, gen.code)))
+                            hidden    = capas.np
+                          ))
                           isolate(modelo)
                         },
+                        
                         rl   = {
                           gen.code       <- codeRl(variable)
                           isolate(modelo <- lm(as.formula(var), 
@@ -239,14 +247,15 @@ mod_ind_nuevos_server <- function(input, output, session, newCases, updateData2,
                           isolate(codedioma$code <- append(codedioma$code,  paste0(sub, gen.code)))
                           isolate(modelo)
                         },
-                        rlr    = {
-                          
+                        
+                        rlr = {
                           scales <- isolate(input$switch.scale.rlr.pred)
                           alpha  <- isolate(input$alpha.rlr.pred)
-                          modelo <- train.glmnet(as.formula(var), data = train, standardize = as.logical(scales), alpha = k, family = "gaussian")
+                          isolate(modelo <- traineR::train.glmnet( as.formula(var), data = train, standardize = as.logical(scales), alpha = alpha, family = "gaussian"))
+                        
+                          gen.code <- codeRlr(variable, alpha, as.logical(scales))
+                          isolate(codedioma$code <- append(codedioma$code, paste0(sub, gen.code)))
                           
-                          gen.code <- codeRlr(variable, alpha,  as.logical(scales))
-                          isolate(codedioma$code <- append(codedioma$code,  paste0(sub, gen.code)))
                           isolate(modelo)
                         },
                         
@@ -254,32 +263,40 @@ mod_ind_nuevos_server <- function(input, output, session, newCases, updateData2,
                           modo.rd  <- isolate(input$mode_rd)
                           scale    <- as.logical(isolate(input$switch.scale.rd.pred))
                           gen.code <- codeRd(variable, modo.rd, scale)
-                          ncomp <- isolate(input$ncomp_rd)
+                          ncomp    <- isolate(input$ncomp_rd)
                           ncompdef <- as.logical(isolate(input$permitir_ncomp))
-                          if(ncompdef) {
-                            if(modo.rd == "ACP") {
+                          form     <- as.formula(var)
+                          datos    <- train
+                          
+                          if (ncompdef) {
+                            if (modo.rd == "ACP") {
                               modelo <- pcr(
-                                formula, ncomp = ncomp, data = datos,
-                                scale = scale, validation = 'CV')
+                                form, ncomp = ncomp, data = datos,
+                                scale = scale, validation = "CV"
+                              )
                             } else {
                               modelo <- plsr(
-                                formula, ncomp = ncomp, data = datos,
-                                scale = scale, validation = 'CV')
+                                form, ncomp = ncomp, data = datos,
+                                scale = scale, validation = "CV"
+                              )
                             }
                           } else {
-                            if(modo.rd == "ACP") {
+                            if (modo.rd == "ACP") {
                               modelo <- pcr(
-                                formula, data = datos,
-                                scale = scale, validation = 'CV')
+                                form, data = datos,
+                                scale = scale, validation = "CV"
+                              )
                             } else {
-                              modelo <- pcr(
-                                formula, data = datos,
-                                scale = scale, validation = 'CV')
+                              modelo <- plsr(
+                                form, data = datos,
+                                scale = scale, validation = "CV"
+                              )
                             }
                           }
                           
-                          optimal.ncomp <- which.min(RMSEP(modelo)$val[1, 1, ]) - 1
+                          optimal.ncomp <- which.min(RMSEP(modelo)$val[, 1, ]) - 1
                           isolate(modelo$optimal.ncomp <- optimal.ncomp)
+                          isolate(codedioma$code <- append(codedioma$code, paste0(sub, gen.code)))
                           isolate(modelo)
                         }
       )
@@ -461,7 +478,12 @@ mod_ind_nuevos_server <- function(input, output, session, newCases, updateData2,
     })
   }, server = F)
   }
-  #Genera la tabla de predicciones
+  
+  # Genera la tabla con los valores predichos por el modelo seleccionado.
+  # Nota: Esta funcion fue modificada para resolver incompatibilidades en modelos especificos
+  # ("boost", "rlr", "rd"), que retornaban sus predicciones en formatos distintos o no compatibles
+  # con el flujo general de la aplicacion. Ahora, la salida de estos modelos se procesa y transforma
+  # adecuadamente para garantizar una visualizacion uniforme y sin errores.
   prediccion <- function(){
     output$PrediTablePN <- DT::renderDataTable({
       input$predecirPromidat
@@ -470,66 +492,116 @@ mod_ind_nuevos_server <- function(input, output, session, newCases, updateData2,
       model<- newCases$modelo
       sel  <- newCases$m.seleccionado
       vari <- newCases$variable.predecir
-      newCases$prediccion        <- NULL
+      newCases$prediccion <- NULL
       tipos  <- c(
         tr("numerico",   isolate(codedioma$idioma)),
         tr("categorico", isolate(codedioma$idioma))
       )
       tryCatch({
-        if(sel == "svm")
-          pred                <- predict(model, test[,-which(colnames(test) == vari)])
-        if(sel == "rd") {
+        if(sel == "svm") {
+          pred <- predict(model, test[, -which(colnames(test) == vari)])
+        } else if(sel == "rlr") {
+          nd <- as.data.frame(test)
+          
+          s_val <- NULL
+          if(!is.null(model$lambda.min) && is.numeric(model$lambda.min)) {
+            s_val <- model$lambda.min
+          } else if(!is.null(model$lambda.1se) && is.numeric(model$lambda.1se)) {
+            s_val <- model$lambda.1se
+          } else if(!is.null(model$bestTune) && !is.null(model$bestTune$lambda)) {
+            s_val <- as.numeric(model$bestTune$lambda)
+          } else if(!is.null(model$lambda) && is.numeric(model$lambda)) {
+            if(!is.null(model$cvm) && length(model$cvm)==length(model$lambda)) {
+              s_val <- model$lambda[ which.min(model$cvm) ]
+            } else {
+              s_val <- model$lambda[1]
+            }
+          }
+          
+          pred_try <- try({
+            if(!is.null(s_val) && is.finite(s_val)) {
+              predict(model, newdata = nd, s = s_val)
+            } else {
+              predict(model, newdata = nd)
+            }
+          }, silent = TRUE)
+          
+          if(inherits(pred_try, "try-error")) {
+            pred_try <- predict(model, newdata = nd)
+          }
+          
+          # Convertir a vector numérico
+          if(is.list(pred_try) && !is.null(pred_try$prediction)) {
+            pred <- as.numeric(pred_try$prediction)
+          } else if(is.matrix(pred_try)) {
+            pred <- as.numeric(pred_try[, 1])
+          } else {
+            pred <- as.numeric(pred_try)
+          }
+        } else if(sel == "rd") {
           ncomp <- model$optimal.ncomp
           auxp  <- predict(model, test, ncomp = ncomp)
-          pred  <- list(prediction = auxp[, , 1], var.pred = variable)
-        }
-        if(sel == "rlr")
+          pred  <- list(prediction = auxp[, , 1], var.pred = vari)
+        } else if(sel == "boost") {
+          ntrees <- isolate(input$iter.boosting.pred)
+          pred <- predict(model, test, n.trees = ntrees)
+        } else {
           pred <- predict(model, test)
-        if(sel %not_in%  c("rlr", "svm", "rd") )
-          pred                <- predict(model, test)
+        }
         
-        datos               <- test
-        if(sel %in%  c("rlr", "rl", "rd", "boost"))
-          datos[,vari]        <- as.numeric(pred)
-        if(sel %not_in%  c("rlr", "rl", "rd", "boost"))
-          datos[,vari]        <- pred$prediction
+        datos <- test
+        
+        if(sel %in% c("rlr", "rl", "rd", "boost")) {
+          if(is.list(pred)) {
+            datos[, vari] <- as.numeric(pred$prediction)
+          } else if(is.matrix(pred)) {
+            datos[, vari] <- as.numeric(pred[, 1])
+          } else {
+            datos[, vari] <- as.numeric(pred)
+          }
+        } else {
+          datos[, vari] <- pred$prediction
+        }
         
         newCases$prediccion <- pred
+        
         nombre.columnas <- c("ID", colnames(datos))
         isolate(codedioma$code <- append(codedioma$code, "predic.nuevos <<- predict(modelo.nuevos, datos.prueba.completos)"))
+        
         tipo.columnas <- sapply(colnames(datos), function(i)
           ifelse(class(datos[,i]) %in% c("numeric", "integer"),
                  paste0("<span data-id='numerico'><i class='fa fa-sort-numeric-up wrapper-tag'></i><br>", tipos[1], "</span>"),
                  paste0("<span data-id='categorico'><i class='fa fa-font wrapper-tag'></i><br>", tipos[2], "</span>")))
+        
         sketch = htmltools::withTags(table(
           tableHeader(nombre.columnas),
           tags$tfoot(
-            tags$tr(tags$th(), lapply(tipo.columnas, function(i) 
+            tags$tr(tags$th(), lapply(tipo.columnas, function(i)
               tags$th(shiny::HTML(i))))
           )
         ))
         DT::datatable(
-          datos, selection = 'none', editable = TRUE,  
+          datos, selection = 'none', editable = TRUE,
           container = sketch, extensions = 'Buttons',
-          options = list(dom = 'Bfrtip', 
-                         buttons = list(list(extend   = "csv", 
-                                             text     = '<i class="fa fa-file-csv"></i>', 
+          options = list(dom = 'Bfrtip',
+                         buttons = list(list(extend   = "csv",
+                                             text     = '<i class="fa fa-file-csv"></i>',
                                              filename = "dataPred",
-                                             exportOptions = list(modifier = list(page = "all"))), 
+                                             exportOptions = list(modifier = list(page = "all"))),
                                         list(extend   = "excel",
-                                             text     = '<i class="fa fa-file-excel"></i>', 
+                                             text     = '<i class="fa fa-file-excel"></i>',
                                              filename = "dataPred",
                                              exportOptions = list(modifier = list(page = "all")))
                          )))
-        
       }, error = function(e) {
         showNotification(paste0("ERROR al mostrar datos: ", e), type = "error")
         return(NULL)
       })
-    }, server = F)}
+    }, server = F)
+  }
   
   
-  #Agrega la predicción a los datos
+  #Agrega la prediccion a los datos
   crear.datos.np <- function(){
     datos.aux.prueba <- newCases$datos.prueba
     datos.aux.prueba[,newCases$variable.predecir]   <- newCases$prediccion$prediction
@@ -592,7 +664,7 @@ mod_ind_nuevos_server <- function(input, output, session, newCases, updateData2,
   })
   
   
-  # Habilitada o deshabilitada el número de componenetes 
+  # Habilitada o deshabilitada el numero de componenetes 
   observeEvent(input$permitir_ncomp, {
     if (as.logical(input$permitir_ncomp)) {
       shinyjs::enable("ncomp_rd")
@@ -677,37 +749,89 @@ mod_ind_nuevos_server <- function(input, output, session, newCases, updateData2,
   })
   
   
+  #Valida que los datos contengan la misma cantidad de columnas 
   #Valida que los datos contengan la misma cantidad de columnas 
   validar <- function() {
-    cod        <- ""
-    originales        <-  newCases$originales
-    datos.aprendizaje <-  newCases$datos.aprendizaje
-    datos.prueba      <-  newCases$datos.prueba
+    originales <- newCases$originales
+    datos.aprendizaje <- newCases$datos.aprendizaje
+    datos.prueba <- newCases$datos.prueba
     
-    tryCatch(
-      for (var in colnames(originales)) {
-        if(var %in% colnames(datos.aprendizaje)) {
-          if(class(datos.prueba[, var]) %not_in% c("numeric", "integer") & 
-             class(datos.aprendizaje[, var]) %in% c("numeric", "integer")) {
-            datos.prueba[, var]       <- as.numeric(datos.prueba[, var])
-          }
-          if(class(datos.prueba[, var]) %in% c("numeric", "integer") & 
-             class(datos.aprendizaje[, var]) %not_in% c("numeric", "integer")) {
-            datos.prueba[, var]       <- as.factor(datos.prueba[, var])
-          }
-        }
-        else{
-          if(paste0(var, ".", unique(originales[, var])[1]) %in% colnames(datos.aprendizaje)){
-            datos.prueba <- loadeR::datos.disyuntivos(datos.prueba, var)
-            datos.prueba[, var]       <- NULL
-          }else{
-            datos.prueba[, var]       <- NULL
-          }
+    tryCatch({
+      # --- Paso 1: Sincronizar columnas ---
+      faltantes <- setdiff(colnames(datos.aprendizaje), colnames(datos.prueba))
+      for (col in faltantes) {
+        if (is.numeric(datos.aprendizaje[, col])) {
+          datos.prueba[, col] <- 0
+        } else {
+          datos.prueba[, col] <- NA
         }
       }
-    )
-    newCases$datos.prueba <- datos.prueba
+      
+      extra <- setdiff(colnames(datos.prueba), colnames(datos.aprendizaje))
+      if (length(extra) > 0) {
+        datos.prueba <- datos.prueba[, !(colnames(datos.prueba) %in% extra)]
+      }
+      
+      datos.prueba <- datos.prueba[, colnames(datos.aprendizaje), drop = FALSE]
+      
+      # --- Paso 2: Igualar tipos y niveles de factores ---
+      for (col in colnames(datos.aprendizaje)) {
+        tipo_train <- class(datos.aprendizaje[, col])
+        tipo_test  <- class(datos.prueba[, col])
+        
+        if (tipo_train != tipo_test) {
+          if (tipo_train %in% c("numeric", "integer")) {
+            datos.prueba[, col] <- as.numeric(datos.prueba[, col])
+          } else if (tipo_train == "factor") {
+            datos.prueba[, col] <- as.factor(datos.prueba[, col])
+          }
+        }
+        
+        if (is.factor(datos.aprendizaje[, col])) {
+          niveles_train <- levels(datos.aprendizaje[, col])
+          datos.prueba[, col] <- factor(datos.prueba[, col], levels = niveles_train)
+        }
+      }
+      
+      # --- Paso 3: Aplicar estructura del model.matrix (solo para redes neuronales) ---
+      if (identical(newCases$m.seleccionado, "nn") && !is.null(newCases$model.matrix.template)) {
+        var <- newCases$variable.predecir
+        form <- as.formula(paste0(var, "~."))
+        
+        # Excluir variable respuesta antes del model.matrix
+        if (var %in% colnames(datos.prueba)) {
+          datos.prueba.sin.y <- datos.prueba[, setdiff(colnames(datos.prueba), var), drop = FALSE]
+        } else {
+          datos.prueba.sin.y <- datos.prueba
+        }
+        
+        # Evitar eliminación de filas con NA
+        mm_test <- model.matrix(~ ., data = datos.prueba.sin.y, na.action = na.pass)
+        mm_test <- as.data.frame(mm_test)
+        
+        
+        # Agregar columnas faltantes
+        faltantes <- setdiff(newCases$model.matrix.template, colnames(mm_test))
+        for (col in faltantes) mm_test[, col] <- 0
+        
+        # Eliminar columnas extra y reordenar
+        extra <- setdiff(colnames(mm_test), newCases$model.matrix.template)
+        if (length(extra) > 0) mm_test <- mm_test[, !(colnames(mm_test) %in% extra)]
+        
+        mm_test <- mm_test[, newCases$model.matrix.template, drop = FALSE]
+        
+        datos.prueba <- mm_test
+      }
+      
+      # --- Paso 4: Guardar resultado final ---
+      newCases$datos.prueba <- datos.prueba
+    },
+    error = function(e) {
+      message("Error al validar datos: ", e$message)
+    })
   }
+  
+  
 }
 
 ## To be copied in the UI
